/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.hop.metadata.api;

public interface IEnumHasCodeAndDescription extends IEnumHasCode {
  String getDescription();

  default int getIndex() {
    IEnumHasCodeAndDescription[] values = getClass().getEnumConstants();
    for (int i = 0; i < values.length; i++) {
      if (values[i].equals(this)) {
        return i;
      }
    }
    throw new RuntimeException("Impossible enum found: " + getCode());
  }

  static <E extends IEnumHasCodeAndDescription> String[] getDescriptions(Class<E> clazz) {
    String[] descriptions = new String[clazz.getEnumConstants().length];
    for (int i = 0; i < descriptions.length; i++) {
      descriptions[i] = clazz.getEnumConstants()[i].getDescription();
    }
    return descriptions;
  }

  static <E extends IEnumHasCodeAndDescription> E lookupDescription(
      Class<E> clazz, String description, E defaultValue) {
    for (E e : clazz.getEnumConstants()) {
      if (e.getDescription().equalsIgnoreCase(description)) {
        return e;
      }
    }
    return defaultValue;
  }
}
